%%
% The following source code is an example of how the
% optical properties of milk are computed using the 
% Lorenz-Mie theory presented in the SIGGRAPH 2007 paper
%
% Computing the Scattering Properties of Participating
% Media Using Lorenz-Mie Theory
%
% By authors: Jeppe Revall Frisvad
%             Niels Joergen Christensen
%             Henrik Wann Jensen
%
% This paper holds full references to the literature 
% mentioned in comments below.
%
% Code written by Jeppe Revall Frisvad, 2013.
% Copyright (c) Jeppe Revall Frisvad 2013
%
% Permission is granted to anyone to use this code as 
% software for any purpose, including commercial applications.
% However, the software is provided 'as-is' without any warranty.
%% 
function [scatter, absorp, extinct, asymmetry] = milk(fat_weight, protein_weight, pressure)
% milk takes the fat and protein contents of a milk product (in g/100 g)
% (and the homogenization pressure in MPa) and returns the optical
% properties of the milk as predicted by the Lorenz-Mie theory.
% Since we neglect interparticle scattering (decoupling approximation),
% the optical properties break if fat_weight > 12 wt.-%.

% Set default to unhomogenized mini (0.5 wt.-%) milk.
if nargin < 3
    pressure = 0;
    if nargin < 2
        protein_weight = 3.4;
        if nargin < 1
            fat_weight = 0.5;
        end
    end
end

% Clear persisitent variables in Lorenz-Mie code
clear B R psi_zeta;

% Data from Hale and Querry [1973] and Pope and Fry [1997]
refrac_water = [ complex(1.341, 3.393e-10)
                 complex(1.339, 2.110e-10)
                 complex(1.338, 1.617e-10)
                 complex(1.337, 3.302e-10)
                 complex(1.336, 4.309e-10)
                 complex(1.335, 8.117e-10)
                 complex(1.334, 1.742e-9)
                 complex(1.333, 2.473e-9)
                 complex(1.333, 3.532e-9)
                 complex(1.332, 1.062e-8)
                 complex(1.332, 1.410e-8)
                 complex(1.331, 1.759e-8)
                 complex(1.331, 2.406e-8)
                 complex(1.331, 3.476e-8)
                 complex(1.330, 8.591e-8)
                 complex(1.330, 1.474e-7)
                 complex(1.330, 1.486e-7) ];

% Absorbance from Du et al. [1998] (PhotochemCAD 1)
abs_riboflavin = [ 0.07117; 0.05934; 0.07215; 0.08300;...
                   0.06379; 0.01962; 0.00950; 0.00957;...
                   0.01198; 0.01047; 0.01046; 0.01061;...
                   0.00671; 0.01034; 0.00589; 0.00886; 0.01015 ];

% Data from Walstra and Jenness [1984] and Michalski et al. [2001]
refrac_fat = [ complex(1.480, 4.0e-6)
               complex(1.476, 6.4e-6)
               complex(1.473, 8.6e-6)
               complex(1.470, 1.14e-5)
               complex(1.468, 1.10e-5)
               complex(1.466, 1.00e-5)
               complex(1.464, 4.7e-6)
               complex(1.463, 4.6e-6)
               complex(1.462, 4.7e-6)
               complex(1.461, 4.9e-6)
               complex(1.460, 5.0e-6)
               complex(1.459, 5.0e-6)
               complex(1.458, 5.1e-6)
               complex(1.457, 5.2e-6)
               complex(1.457, 5.2e-6)
               complex(1.456, 5.2e-6)
               complex(1.456, 5.2e-6) ];

% Constants
NO_SAMPLES = 17;
SQRT_2PI = sqrt(2*pi);

% Allocate memory for results
scatter = zeros(NO_SAMPLES, 1);
absorp = zeros(NO_SAMPLES, 1);
extinct = zeros(NO_SAMPLES, 1);
asymmetry = zeros(NO_SAMPLES, 1);

% Densities of different milk constituents [Walstra and Jenness 1984]
fat_density = 0.915;    % Milk fat density (g/mL)
casein_density = 1.11;  % Density of hydrated casein (g/mL)
  
% Formula to get the total density as a function of the fat content [Walstra and Jenness 1984]
milk_density = 1/(1.23e-3*fat_weight + 0.9665);

% Conversion from absorbance to absorption coefficients.
% Scale based on molar absorption coefficient: 33000 M^-1 cm^-1 for 266.5 nm [Koziol 1966]
scale = 33.0e5/0.25654;
for i = 1:NO_SAMPLES
    abs_riboflavin(i) = scale*abs_riboflavin(i);
end

% Weight fraction of riboflavin in milk [Fox and McSweeney 1998].
% Natural content: 0.17 mg pr. 100 g.
% Molar mass of riboflavin (C_17 H_20 N_4 O_6): 376.3682 g/mol
w_riboflavin = log(10)*(0.17/376.3682)/(100/milk_density);

% Correction of host medium refractive index
for i = 1:NO_SAMPLES
    wavelength = 350e-9 + i*25e-9;
    refrac_water(i) = refrac_water(i) + complex(0, w_riboflavin*abs_riboflavin(i)*wavelength/(4*pi));
end

% Functional expresions finding fat globule size distribution
log_50 = log10(50);
v_sqr = fat_weight*fat_weight;
r_P50 = 7.481403922e-4*v_sqr + 3.203991829e-2*fat_weight + 0.1162155667;
if fat_weight < 2
    r_P0 = -0.2528033794*v_sqr + 1.418606759*fat_weight;
else
    r_P0 = 1.456189142*fat_weight^0.36;
end
if fat_weight > 0
    beta = log10(r_P0);
else
    beta = 0;
end
alpha = beta/log_50 - log10(r_P50)/log_50;

cv = 0.6;
if pressure > 0
    fat_r_vs = 10^(-alpha*log10(pressure) + beta)/(cv*cv + 1);
else
    fat_r_vs = r_P0/(cv*cv + 1);
end
fat_c_s = fat_r_vs*cv;
fat_r_vs = fat_r_vs*1e-6;

% More input data [Walstra 1975, Schmidt et al. 1973]:
fat_r_min = 0.005e-6;
fat_r_max = 10.0e-6;
fat_dr = 0.05e-6;
casein_r_vs = 43.0e-9;
casein_c_s = 0.23;
casein_r_min = 0.0e-9;
casein_r_max = 150.0e-9;
casein_dr = 1.0e-9;
casein_refrac_idx =  complex(1.503, 0); % [Attaie and Richtert 2000]

fat_conc = fat_weight/(100/milk_density);
fat_volume = fat_conc/fat_density;
casein_conc = 0.76*protein_weight/(100/milk_density);
casein_volume = casein_conc/casein_density;
str = sprintf('Fat volume fraction: %e', fat_volume);
disp(str);
str = sprintf('Casein volume fraction: %e\n', casein_volume);
disp(str);

% Particle size distributions
fat_N = zeros(int32((fat_r_max - fat_r_min)/fat_dr) + 1, 1);

if fat_volume <= 0.0
    alpha = 0;
    beta = 1;
else
    beta_sqr = log(fat_c_s*fat_c_s/(fat_r_vs*fat_r_vs*1e12) + 1);
    alpha = log(fat_r_vs*1e6) - 0.5*beta_sqr;
    beta = sqrt(beta_sqr);
end

volume = 0;
counter = 1;
for r = (fat_r_min + fat_dr*0.5):fat_dr:fat_r_max 
    tmp = (log(r*1e6) - alpha)/beta;
    fat_N(counter) = 1.0/(r*1e6*beta*SQRT_2PI)*exp(-0.5*tmp*tmp);
    volume = volume + fat_N(counter);
    counter = counter + 1;
end
fat_N = fat_N*(fat_volume/volume);

counter = 1;
for r = (fat_r_min + fat_dr*0.5):fat_dr:fat_r_max
    fat_N(counter) = fat_N(counter)*3/(4*pi*r*r*r)/fat_dr;
    counter = counter + 1;
end
%str = sprintf('Total number density of fat globules: %e', sum(fat_N)*(fat_r_max - fat_r_min));
%disp(str);

casein_N = zeros(int32((casein_r_max - casein_r_min)/casein_dr) + 1, 1);

x_vs = casein_r_vs*1e9/(150 - casein_r_vs*1e9);
beta_sqr = log(casein_c_s*casein_c_s/(x_vs*x_vs) + 1);
alpha = log(x_vs) - 0.5*beta_sqr;
beta = sqrt(beta_sqr);

volume = 0;
counter = 1;
for r = (casein_r_min + casein_dr*0.5):casein_dr:casein_r_max
    x = r*1.0e9/(150.0 - r*1.0e9);
    tmp = (log(x) - alpha)/beta;
    casein_N(counter) = 1/(x*beta*SQRT_2PI)*exp(-0.5*tmp*tmp);
    volume = volume + casein_N(counter);
    counter = counter + 1;
end
casein_N = casein_N*casein_volume/volume;

counter = 1;
for r = (casein_r_min + casein_dr*0.5):casein_dr:casein_r_max
    casein_N(counter) = casein_N(counter)*3/(4*pi*r*r*r)/casein_dr;
    counter = counter + 1;
end
%str = sprintf('Total number density of casein micelles: %e', sum(casein_N)*(casein_r_max - casein_r_min));
%disp(str);

% Optical properties
for i = 1:NO_SAMPLES
    wavelength = 350e-9 + i*25e-9;
    str = sprintf('Wavelength: %0.3e (idx: %d)', wavelength, i);
    disp(str);
    str = sprintf('Refractive index of water: %f + i%e', real(refrac_water(i)), imag(refrac_water(i)));
    disp(str);

    fat_ext = 0;
    fat_sca = 0;
    fat_abs = 0;
    fat_g = 0;
    counter = 1;
    for r = (fat_r_min + fat_dr*0.5):fat_dr:fat_r_max
        [C_t, C_s, C_a, g] = particle_props(...
            r, wavelength, refrac_fat(i), refrac_water(i));

        sigma_s = C_s*fat_N(counter)*fat_dr;
        fat_ext = fat_ext + C_t*fat_N(counter)*fat_dr;
        fat_sca = fat_sca + sigma_s;
        fat_abs = fat_abs + C_a*fat_N(counter)*fat_dr;
        fat_g = fat_g + g*sigma_s;
        counter = counter + 1;
    end
    if fat_sca > 0
        fat_g = fat_g/fat_sca;
    end

    casein_ext = 0;
    casein_sca = 0;
    casein_abs = 0;
    casein_g = 0;
    counter = 1;
    for r = (casein_r_min + casein_dr*0.5):casein_dr:casein_r_max
        [C_t, C_s, C_a, g] = particle_props(...
            r, wavelength, casein_refrac_idx, refrac_water(i));

        sigma_s = C_s*casein_N(counter)*casein_dr;
        casein_ext = casein_ext + C_t*casein_N(counter)*casein_dr;
        casein_sca = casein_sca + sigma_s;
        casein_abs = casein_abs + C_a*casein_N(counter)*casein_dr;
        casein_g = casein_g + g*sigma_s;
        counter = counter + 1;
    end
    if casein_sca > 0.0
        casein_g = casein_g/casein_sca;
    end
    
    water_absorp = 4*pi*imag(refrac_water(i))/wavelength;
    if casein_sca - casein_ext > water_absorp
        exit;
    end

    extinct(i) = fat_ext + casein_ext + water_absorp;
    scatter(i) = fat_sca + casein_sca;
    absorp(i) = extinct(i) - scatter(i);
    if scatter(i) > 0
      asymmetry(i) = (fat_sca*fat_g + casein_sca*casein_g)/scatter(i);
    end

    str = sprintf('Extinction coefficient (%0.1fnm): %0.2f [1/m]', wavelength*1e9, extinct(i));
    disp(str);
    str = sprintf('Scattering coefficient (%0.1fnm): %0.2f [1/m]', wavelength*1e9, scatter(i));
    disp(str);
    str = sprintf('Absorption coefficient (%0.1fnm): %0.2f [1/m]', wavelength*1e9, absorp(i));
    disp(str);
    str = sprintf('Ensemble asymmetry parameter (%0.1fnm): %0.6f\n', wavelength*1e9, asymmetry(i));
    disp(str);
end
end

